package main

import (
	"crypto/tls"
	"flag"
	"log"
	"os"
	"path"
)

func init() {
	log.SetOutput(os.Stdout)
}

var verbose bool

func main() {
	configFile := flag.String("config", "", "path to configuration file")
	flag.BoolVar(&verbose, "verbose", false, "print request and response information")
	flag.Parse()

	if *configFile == "" {
		homedir, err := os.UserHomeDir()
		if err == nil && homedir != "" {
			*configFile = path.Join(homedir, ".config", "twins", "config.yaml")
		}
	}

	err := readconfig(*configFile)
	if err != nil {
		log.Fatalf("failed to read configuration file at %s: %v\nSee CONFIGURATION.md for information on configuring twins", *configFile, err)
	}

	if config.hostname == "" || config.port <= 0 {
		log.Fatal("hostname and port must be specified")
	}

	if len(config.Certificates) == 0 {
		log.Fatal("at least one certificate must be specified (gemini requires TLS for all connections)")
	}

	var certificates []tls.Certificate
	for _, cert := range config.Certificates {
		cert, err := tls.LoadX509KeyPair(cert.Cert, cert.Key)
		if err != nil {
			log.Fatalf("failed to load certificate: %s", err)
		}
		certificates = append(certificates, cert)
	}

	log.Printf("twins running on %s:%d", config.hostname, config.port)

	listen(config.Listen, certificates)
}
